import { animationPromise, cyclicArray, Alarms, html } from "./utils";

const items = [
	{
		img:
			"https://a.allegroimg.com/original/110ef3/20ea48304d19b18e409748a8fdc1/KOMIN-NA-TWARZ-ZAMIAST-MASECZKI-BAWELNA",
		name: "lorem ip sum! 1 ",
	},
	{
		img: "https://i.ytimg.com/vi/ggrm3Sn1HGw/maxresdefault.jpg",
		name: "lorem ip sum! 2 ",
	},
	{
		img:
			"https://www.freetutorialshub.com/wp-content/uploads/2020/04/learn-css-transition-and-animation_5e919fd2f0264.jpeg",
		name: "lorem ip sum! 3 ",
	},
	{
		img:
			"https://a.allegroimg.com/original/110ef3/20ea48304d19b18e409748a8fdc1/KOMIN-NA-TWARZ-ZAMIAST-MASECZKI-BAWELNA",
		name: "lorem ip sum! 4",
	},
];

class Carousel extends HTMLElement {
	items = cyclicArray(items);
	rotatating = false;
	shadow;

	constructor() {
		super();
		this.shadow = this.attachShadow({ mode: "open" });
	}

	connectedCallback() {
		this.shadow.innerHTML = html`
			<button class="next icon-left-open"></button>
			<div class="container">
				<img src="${this.items.next().value.img}" />
			</div>
			<button class="prev icon-right-open"></button>
			<style>
				${Carousel.css}
			</style>
		`;

		const next = this.shadow.querySelector(".next") as HTMLButtonElement;
		const prev = this.shadow.querySelector(".prev") as HTMLButtonElement;
		next.addEventListener("click", () => this.rotate());
		prev.addEventListener("click", () => this.rotate(true));
		Alarms.create(6000, "rotateAlarm").then(() => this.rotate());
	}

	rotate(reverse = false) {
		Alarms.create(6000, "rotateAlarm").then(() => this.rotate());
		if (this.rotatating) return;

		const container = this.shadow.querySelector(".container") as HTMLDivElement;
		const oldItem = container.querySelector(
			"img:last-of-type"
		) as HTMLImageElement;
		const newItem = document.createElement("img") as HTMLImageElement;

		const inClass = reverse ? ["reverse", "left"] : ["right"];
		const outClass = reverse ? ["reverse", "right"] : ["left"];

		//Sanity checks
		this.shadow
			.querySelectorAll("." + inClass.join("."))
			.forEach((e) => e.remove());

		newItem.src = this.items.next(reverse).value.img;
		container.appendChild(newItem);

		animationPromise(oldItem).finally(() => container.removeChild(oldItem));

		oldItem.classList.add(...outClass);

		animationPromise(newItem).finally(() => {
			newItem.classList.remove(...inClass);
			this.rotatating = false;
		});
		newItem.classList.add(...inClass);
		this.rotatating = true;
	}

	static get css() {
		return `
			:host {
				width: 100%;
				height: 100%;
				display: flex;
				align-items: center;
				justify-content: center;
			}

			.container {
				height: 100%;
				width: 60%;
				clip-path: margin-box;
				position: relative;
				display: flex;
				align-items: center;
				justify-content: center;
				overflow: hidden;
			}

			img {
				animation-duration: 1.5s;
				object-fit: scale-down;
				width: 100%;
				height: 100%;
			}

			@keyframes slide-to-left {
				from {
					transform: translateX(0%); 	
				}
				to {
					transform: translateX(-200%); 	
				}
			}

			@keyframes slide-from-right {
				from {
					transform: translateX(200%); 	
				}
				to {
					transform: translateX(0%); 	
				}
			}

			.right {
				animation-name: slide-from-right;
			}

			.left {
				animation-name: slide-to-left;
				position: absolute;
			}

			.reverse {
				animation-direction: reverse;
			}

			.left.reverse {
				position: static; 
			}

			.right.reverse {
				position: absolute; 
			}

			button {
				border: none;
				border-radius: 50%;
				height: 2.2em;
				width: 2.2em;
				background-color: var(--background-opposite-4);
				cursor: pointer;
				z-index: 1;
				color: var(--background-primary-3);
				padding: 0;
				margin: -20%;
			}

[class^="icon-"]:before, [class*=" icon-"]:before {
  font-family: "fontello";
  font-style: normal;
  font-weight: normal;
  speak: never;
 
  display: inline-block;
  text-decoration: inherit;
  width: 1em;
  margin-right: .2em;
  text-align: center;
  /* opacity: .8; */
 
  /* For safety - reset parent styles, that can break glyph codes*/
  font-variant: normal;
  text-transform: none;
 
  /* fix buttons height, for twitter bootstrap */
  line-height: 1em;
 
  /* Animation center compensation - margins should be symmetric */
  /* remove if not needed */
  margin-left: .2em;
 
  /* you can be more comfortable with increased icons size */
  /* font-size: 120%; */
 
  /* Font smoothing. That was taken from TWBS */
  -webkit-font-smoothing: antialiased;
  -moz-osx-font-smoothing: grayscale;
 
  /* Uncomment for 3D effect */
  /* text-shadow: 1px 1px 1px rgba(127, 127, 127, 0.3); */
}
 
.icon-left-open:before { content: '\\e800'; } /* '' */
.icon-right-open:before { content: '\\e801'; } /* '' */
		`;
	}
}
customElements.define("x-carousel", Carousel);
